@@ -16,6 +16,19 @@ import re
 mutable_default_args = re.compile(r"^\s*def .+\((.+=\{\}|.+=\[\])")
 
 
+def no_log_warn(logical_line):
+    """Disallow 'LOG.warn('
+
+    Deprecated LOG.warn(), instead use LOG.warning
+    https://bugs.launchpad.net/senlin/+bug/1508442
+    N352
+    """
+
+    msg = ("N352: LOG.warn is deprecated, please use LOG.warning!")
+    if "LOG.warn(" in logical_line:
+        yield (0, msg)
+
+
 def no_mutable_default_args(logical_line):
     msg = "N301: Method's default argument shouldn't be mutable!"
     if mutable_default_args.match(logical_line):
@@ -24,3 +37,4 @@ def no_mutable_default_args(logical_line):
 
 def factory(register):
     register(no_mutable_default_args)
+    register(no_log_warn)
